module FileSystemLayerDelete

/********************************************************************
 * authors: 
 * 	Miguel Ferreira		<miguel@di.uminho.pt>
 * 	Samuel Silva   		<pg11034@alunos.uminho.pt>
 * status: experimental (unstable, experimental, stable)
 * description: 
 * 		
 */


open RelCalc
open FileSystemLayerBase
open FlashFileSystemCore

/********************************************************************
 * DeleteFileDir: page 30
 */
-- main funcion, which is totaly defined
pred FS_DeleteFileDir_Main[sys, sys': System, full_path:Path, status: FFS_Status] {
  (full_path in RelCalc/dom[sys.fileStore.map] and
   pre_FS_DeleteFileDir_System[sys,full_path])
  => (FS_DeleteFileDir_System[sys,sys',full_path] and 
      status = FFS_StatusSuccess)
     else
     (FS_DeleteFileDir_Exception[sys,full_path,status] and
	  sys' = sys)
}


-- system level function
pred FS_DeleteFileDir_System[sys, sys': System, full_path: Path] {
  FS_DeleteFileDir_FileStore[sys.fileStore,sys'.fileStore,{full_path}] and 
  sys.table = sys'.table
}

pred pre_FS_DeleteFileDir_System[sys: System, full_path: Path] {
  full_path not in RelCalc/rng[sys.table.map].path
  pre_FS_DeleteFileDir_FileStore[sys.fileStore,{full_path}]
}


-- file store level function
pred FS_DeleteFileDir_FileStore[fs,fs': FileStore, paths: set Path] {
  fs'.map = fs.map - (paths -> (paths.(fs.map)))
}

pred pre_FS_DeleteFileDir_FileStore[fs: FileStore, paths: set Path] {
  (((Path - paths)->Path) & iden).(fs.map) in dirName.((Path - paths)->File)
}


-- exception function (error type detection)
pred FS_DeleteFileDir_Exception[sys: System, full_path: Path, status: FFS_Status]{
  not isElemFileStore[full_path,sys.fileStore]
  => status = FS_ErrorFileNotFound
     else (isElemTable[full_path,sys.table]
	       => status = FS_ErrorFileStillOpen
		      else ((isDirectory[sys.fileStore.map[full_path]] and
                     hasSubFiles[sys.fileStore,full_path])
					=> status = FS_ErrorDirectoryNonEmpty
					   else status = FFS_StatusUnknown))
}



/***********************************************/
-- Show predicates --

-- Main --

pred ShowFS_DeleteFileDir_Main[sys,sys': System, full_path: Path, status: FFS_Status] {
  OpenFilesTable = sys.table + sys'.table                                 and
  File = RelCalc/rng[sys.fileStore.map] + RelCalc/rng[sys'.fileStore.map] and
  SystemInvariantVDM[sys]                                                 and
  FS_DeleteFileDir_Main[sys,sys',full_path,status]                        and
  sys not = sys'
}

ShowDeleteFileDir_Main: run ShowFS_DeleteFileDir_Main
                        for 3 but 2 System,
						          1 FFS_Status


-- System --

pred ShowFS_DeleteFileDir_System[sys,sys': System, full_path: Path] {
  OpenFilesTable = sys.table + sys'.table                                 and
  Path = RelCalc/dom[sys.fileStore.map] + RelCalc/dom[sys'.fileStore.map] and
  File = RelCalc/rng[sys.fileStore.map] + RelCalc/rng[sys'.fileStore.map] and
  SystemInvariantVDM[sys]                                                 and
  pre_FS_DeleteFileDir_System[sys,full_path]                              and
  FS_DeleteFileDir_System[sys,sys',full_path]                             and
  sys not = sys'
}

ShowDeleteFileDir_System: run ShowFS_DeleteFileDir_System
                          for 3 but 2 System

-- FileStore --

pred ShowFS_DeleteFileDir_FileStore[fs,fs': FileStore, full_path: set Path] {
--  PathInvariantVDM[Path]        and        
  Path = RelCalc/dom[fs.map] + RelCalc/dom[fs'.map] and
  File = RelCalc/rng[fs.map] + RelCalc/rng[fs'.map] and
  some fs.map                                       and
  FileStoreInvariantVDM[fs]                         and
  pre_FS_DeleteFileDir_FileStore[fs,full_path]      and
  FS_DeleteFileDir_FileStore[fs,fs',full_path]      and
  fs not = fs'
}

ShowDeleteFileDir_FileStore: run ShowFS_DeleteFileDir_FileStore 
                             for 3 but 2 FileStore,
                                       0 System,
								       0 OpenFilesTable


-- Exceptions --

pred ShowFS_DeleteFileDir_Exception[sys:System, full_path: Path, status: FFS_Status] {
  SystemInvariantVDM[sys]     and
  PathInvariantVDM[full_path] and
  FS_DeleteFileDir_Exception[sys,full_path,status]
}


pred ShowFS_ErrorFileNotFound[sys: System, full_path: Path,status: FFS_Status] {
  ShowFS_DeleteFileDir_Exception[sys,full_path,status] and
  status = FS_ErrorFileNotFound
}

ShowFileNotFound: run ShowFS_ErrorFileNotFound 
                  for 3 but 1 System,
                            1 FFS_Status

pred ShowFS_ErrorFileStillOpen[sys: System, full_path: Path,status: FFS_Status] {
  ShowFS_DeleteFileDir_Exception[sys,full_path,status] and
  status = FS_ErrorFileStillOpen
}

ShowFileStillOpen: run ShowFS_ErrorFileStillOpen 
                   for 3 but 1 System,
                             1 FFS_Status

pred ShowFS_ErrorDirectoryNonEmpty[sys: System, full_path: Path,status: FFS_Status] {
  ShowFS_DeleteFileDir_Exception[sys,full_path,status] and
  status = FS_ErrorDirectoryNonEmpty
}

ShowDirectoryNonEmpty: run ShowFS_ErrorDirectoryNonEmpty 
                       for 3 but 1 System,
                                 1 FFS_Status
